# This takes the output of the `userspace` derivation, plus the
# modules from the `kernel` derivation, and creates an
# initramfs-formatted `cpio` archive from their contents.
{ nixpkgsOnBuildForHost
, nixpkgsOnBuildForBuild
, lib
, userspace
, kernel

# a list of paths (relative to ${kernel}/lib/modules/*/kernel) to modules .ko
# files which should be included in the initrd
, modules ? [ ]
}:

nixpkgsOnBuildForHost.stdenv.mkDerivation {
  name = "initramfs.cpio";
  nativeBuildInputs = with nixpkgsOnBuildForBuild; [ findutils cpio ];
  srcs = [ ];
  dontUnpack = true;
  modulesList = (lib.concatMapStringsSep "\n" (m: "${kernel.version}/kernel/" + m) modules)+"\n";
  passAsFile = [ "modulesList" ];
  buildPhase = ''
    mkdir build
    mkdir -p build/lib/modules
    BUILD=$(pwd)/build
    pushd ${kernel}/lib/modules/
      cat $modulesListPath | cpio -p -d $BUILD/lib/modules
      popd
    pushd ${userspace}
      find . | cpio -p -d $BUILD/
      popd
    chmod -R u+w $BUILD
    pushd $BUILD
      find . | cpio --create -H newc -R +0:+0 > $out
      popd
  '';
  dontInstall = true;

  passthru = {
    inherit modules;
  };
}
